//
//  Created by Ingvar Nedrebo on Mon Mar 18 2002.
//
//
// This is a very simple example of how to call an interactive
// command line tool from Cocoa using Pseudo Terminals and NSTask.
//
// First, a pseudo tty (pty) is created using the PseudoTTY class.
// Then an NSTask is created with the client (slave) side of the pty
// as standard input, output and error.
// The program then forwards all key presses in the text view to
// the shell, and uses background reading to get all the output
// from the shell.
//
// This example is very similar in structure to Apple's Moriarity
// sample project; you should refer to that for explanation of
// the concepts in this example, as Apple's example is well commented.


#import "PtyView.h"
#import "PseudoTTY.h"
#import <unistd.h>
#import <util.h>

@interface PtyView (PtyPrivate)
-(void)startTask;
-(void) didRead: (NSNotification *)fileNoty;
@end

@implementation PtyView

- (id)initWithFrame:(NSRect)frame;
{
    if (self = [super initWithFrame: frame])
    {
        [self setFont:[NSFont fontWithName:@"Courier" size:0.0]];
        [self startTask];
    }
    return self;
}

- (void)keyDown:(NSEvent *)event
{
    const char * typein = [[event characters] UTF8String];
    [[pty_ masterFileHandle]
          writeData:[NSData dataWithBytes:typein length:strlen(typein)]];
}

@end


@implementation PtyView (PtyPrivate)

-(void)startTask
{
    NSString * cmd = @"/bin/sh";
    pty_ = [[PseudoTTY alloc] init];

    NSTask * task = [[NSTask alloc] init];
    [task setStandardInput:[pty_ slaveFileHandle]];
    [task setStandardOutput:[pty_ slaveFileHandle]];
    [task setStandardError:[pty_ slaveFileHandle]];

    [task setCurrentDirectoryPath:[@"~" stringByExpandingTildeInPath]];
    [task setLaunchPath:cmd];

    [[NSNotificationCenter defaultCenter]
                addObserver:self
                   selector:@selector(didRead:)
                       name:NSFileHandleReadCompletionNotification
                     object:[pty_ masterFileHandle]];

    [[pty_ masterFileHandle] readInBackgroundAndNotify];

    [task launch];

    [self insertText:
        [NSString stringWithFormat:@"Started %@ on terminal %@", cmd, [pty_ name]]];

}

-(void) didRead: (NSNotification *)noty
{
    NSData * data = [[noty userInfo] objectForKey:NSFileHandleNotificationDataItem];

    if ([data length] == 0)
        return; // end of file

    NSString * str = [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding];
    [self insertText:str];
    [str release];

    [[noty object] readInBackgroundAndNotify];
}

@end
